package co.kr.ajinpaper.salesTask;

import androidx.activity.OnBackPressedCallback;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.FileProvider;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.ClipData;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class MainActivity extends AppCompatActivity {
    private ValueCallback<Uri[]> mUploadMessage;
    private Uri mCapturedImageURI = null;
    private PermissionManager permission;
    private long lastBackPressedTime = 0; // 마지막으로 뒤로 가기 버튼이 눌린 시간을 기록하는 변수
    private static final int BACK_PRESS_INTERVAL = 2000; // 2초 간격

    @SuppressLint("SetJavaScriptEnabled")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        WebView myWebView = (WebView) findViewById(R.id.webView);

        permissionCheck();

        if (getIntent().getExtras() != null) {
            String title = getIntent().getStringExtra("title");
            Log.d("MainActivity", "Title: " + title);

            SharedPreferences sharedPreferences = getSharedPreferences("MyApp", MODE_PRIVATE);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putString("pushTitle", title).apply();
        }

        // 뒤로 가기 버튼으로 뒤로 가기 구현
        OnBackPressedCallback callback = new OnBackPressedCallback(true) {
            @Override
            public void handleOnBackPressed() {
                if (myWebView.canGoBack()) {
                    myWebView.goBack();
                } else {
                    if (lastBackPressedTime + BACK_PRESS_INTERVAL > System.currentTimeMillis()) {
                        finish();
                    } else {
                        Toast.makeText(MainActivity.this, "뒤로 가기 버튼을 한 번 더 누르면 종료됩니다.", Toast.LENGTH_SHORT).show();
                    }
                    lastBackPressedTime = System.currentTimeMillis();
                }
            }
        };
        getOnBackPressedDispatcher().addCallback(this, callback);

        WebSettings webSettings = myWebView.getSettings();
        webSettings.setJavaScriptEnabled(true);
        webSettings.setDomStorageEnabled(true);
        webSettings.setAllowFileAccess(true);
        webSettings.setAllowContentAccess(true);
        webSettings.setLoadsImagesAutomatically(true);
        webSettings.setSupportMultipleWindows(true);

        myWebView.addJavascriptInterface(new WebAppInterface(this), "AndroidInterface");

        // 웹뷰에서 파일 입력이 가능하게 바꾸기
        myWebView.setWebChromeClient(new WebChromeClient() {
            // For Android 5.0+
            @SuppressLint("QueryPermissionsNeeded")
            public boolean onShowFileChooser(WebView view, ValueCallback<Uri[]> filePath, FileChooserParams fileChooserParams) {
                if (mUploadMessage != null) {
                    mUploadMessage.onReceiveValue(null);
                }

                mUploadMessage = filePath;

                Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                if (takePictureIntent.resolveActivity(getPackageManager()) != null) {
                    File photoFile = null;
                    photoFile = createImageFile();
                    if (mCapturedImageURI != null) {
                        takePictureIntent.putExtra("PhotoPath", mCapturedImageURI.toString());
                    }

                    if (photoFile != null) {
                        mCapturedImageURI = FileProvider.getUriForFile(getApplicationContext(), getApplicationContext().getPackageName() + ".provider", photoFile);
                        takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, mCapturedImageURI);
                    } else {
                        takePictureIntent = null;
                    }
                }

                Intent contentSelectionIntent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
                contentSelectionIntent.addCategory(Intent.CATEGORY_OPENABLE);
                contentSelectionIntent.setType("image/*");
                contentSelectionIntent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);

                Intent[] intentArray;
                if (takePictureIntent != null) {
                    intentArray = new Intent[]{takePictureIntent};
                } else {
                    intentArray = new Intent[0];
                }

                Intent chooserIntent = new Intent(Intent.ACTION_CHOOSER);
                chooserIntent.putExtra(Intent.EXTRA_INTENT, contentSelectionIntent);
                chooserIntent.putExtra(Intent.EXTRA_TITLE, "Image Chooser");
                chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS, intentArray);

                mFileChooser.launch(chooserIntent);

                return true;
            }
        });


        myWebView.loadUrl("http://165.229.169.31:93"); // 로컬 서버 지정 중. 웹이 실제 서버에 업로드되면 수정해야함.

    }

    // 이미지 파일 등록 가능하도록
    private final ActivityResultLauncher<Intent> mFileChooser = registerForActivityResult(
            new ActivityResultContracts.StartActivityForResult(),
            result -> {
                if (mUploadMessage == null) {
                    return;
                }

                Uri[] results = null;

                if (result.getResultCode() == Activity.RESULT_OK) {
                    if (result.getData() == null || (result.getData().getData() == null && result.getData().getClipData() == null)) {
                        if (mCapturedImageURI != null) {
                            results = new Uri[]{mCapturedImageURI};
                        }
                    } else {
                        String dataString = result.getData().getDataString();
                        ClipData clipData = result.getData().getClipData();
                        if (clipData != null) {
                            results = new Uri[clipData.getItemCount()];
                            for (int i = 0; i < clipData.getItemCount(); i++) {
                                ClipData.Item item = clipData.getItemAt(i);
                                results[i] = item.getUri();
                            }
                        }

                        if (dataString != null) {
                            results = new Uri[]{Uri.parse(dataString)};
                        }
                    }
                }

                mUploadMessage.onReceiveValue(results);
                mUploadMessage = null;
            }
    );

    private File createImageFile() {
        try {
            @SuppressLint("SimpleDateFormat") String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
            String imageFileName = "JPEG_" + timeStamp + "_";
            File storageDir = getExternalFilesDir(Environment.DIRECTORY_PICTURES);
            File imageFile = File.createTempFile(imageFileName, ".jpg", storageDir);
            mCapturedImageURI = Uri.fromFile(imageFile);
            return imageFile;
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        }
    }

    // 권함 알림 메소드
    private void permissionCheck(){
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            permission = new PermissionManager(this, this);

            if(!permission.checkPermission()){
                permission.requestPermission();
            }
        }
    }
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            if (!permission.permissionResult(requestCode, permissions, grantResults)) {
                Toast.makeText(this, "권한 요청이 거절되어 푸시를 받을 수 없습니다.", Toast.LENGTH_SHORT).show();
            }
        }
    }

}